@@ -1,9 +1,12 @@
 """Helper Classes for use of automatic differentiation."""
-from .. import get_backend, exceptions
-from .common import shim
-
 import logging
 
+import numpy as np
+
+from pyhf import exceptions
+from pyhf.optimize.common import shim
+from pyhf.tensor.manager import get_backend
+
 log = logging.getLogger(__name__)
 
 
@@ -29,11 +32,23 @@ class OptimizerMixin:
             )
 
     def _internal_minimize(
-        self, func, x0, do_grad=False, bounds=None, fixed_vals=None, options={}
+        self,
+        func,
+        x0,
+        do_grad=False,
+        bounds=None,
+        fixed_vals=None,
+        options={},
+        par_names=None,
     ):
 
         minimizer = self._get_minimizer(
-            func, x0, bounds, fixed_vals=fixed_vals, do_grad=do_grad
+            func,
+            x0,
+            bounds,
+            fixed_vals=fixed_vals,
+            do_grad=do_grad,
+            par_names=par_names,
         )
         result = self._minimize(
             minimizer,
@@ -69,6 +84,13 @@ class OptimizerMixin:
         if uncertainties is not None:
             # extract number of fixed parameters
             num_fixed_pars = len(fitted_pars) - len(fitresult.x)
+
+            # FIXME: Set uncertainties for fixed parameters to 0 manually
+            # https://github.com/scikit-hep/iminuit/issues/762
+            # https://github.com/scikit-hep/pyhf/issues/1918
+            # https://github.com/scikit-hep/cabinetry/pull/346
+            uncertainties = np.where(fitresult.minuit.fixed, 0.0, uncertainties)
+
             # stitch in zero-uncertainty for fixed values
             uncertainties = stitch_pars(
                 tensorlib.astensor(uncertainties),
@@ -157,7 +179,21 @@ class OptimizerMixin:
             do_stitch=do_stitch,
         )
 
-        result = self._internal_minimize(**minimizer_kwargs, options=kwargs)
+        # handle non-pyhf ModelConfigs
+        try:
+            par_names = pdf.config.par_names
+        except AttributeError:
+            par_names = None
+
+        # need to remove parameters that are fixed in the fit
+        if par_names and do_stitch and fixed_vals:
+            for index, _ in fixed_vals:
+                par_names[index] = None
+            par_names = [name for name in par_names if name]
+
+        result = self._internal_minimize(
+            **minimizer_kwargs, options=kwargs, par_names=par_names
+        )
         result = self._internal_postprocess(
             result, stitch_pars, return_uncertainties=return_uncertainties
         )
