@@ -1,6 +1,6 @@
 """Minuit Optimizer Class."""
-from .. import exceptions
-from .mixins import OptimizerMixin
+from pyhf import exceptions
+from pyhf.optimize.mixins import OptimizerMixin
 import scipy
 import iminuit
 
@@ -18,10 +18,11 @@ class minuit_optimizer(OptimizerMixin):
 
         .. note::
 
-            ``errordef`` should be 1.0 for a least-squares cost function and 0.5
-            for negative log-likelihood function. See page 37 of
-            http://hep.fi.infn.it/minuit.pdf. This parameter is sometimes
-            called ``UP`` in the ``MINUIT`` docs.
+            ``errordef`` should be 1.0 for a least-squares cost function and 0.50
+            for negative log-likelihood function --- see `MINUIT: Function Minimization
+            and Error Analysis Reference Manual <https://cdsweb.cern.ch/record/2296388/>`_
+            Section 7.1: Function normalization and ERROR DEF.
+            This parameter is sometimes called ``UP`` in the ``MINUIT`` docs.
 
 
         Args:
@@ -40,17 +41,21 @@ class minuit_optimizer(OptimizerMixin):
         super().__init__(*args, **kwargs)
 
     def _get_minimizer(
-        self, objective_and_grad, init_pars, init_bounds, fixed_vals=None, do_grad=False
+        self,
+        objective_and_grad,
+        init_pars,
+        init_bounds,
+        fixed_vals=None,
+        do_grad=False,
+        par_names=None,
     ):
 
-        step_sizes = [(b[1] - b[0]) / float(self.steps) for b in init_bounds]
         fixed_vals = fixed_vals or []
         # Minuit wants True/False for each parameter
         fixed_bools = [False] * len(init_pars)
         for index, val in fixed_vals:
             fixed_bools[index] = True
             init_pars[index] = val
-            step_sizes[index] = 0.0
 
         # Minuit requires jac=callable
         if do_grad:
@@ -60,8 +65,7 @@ class minuit_optimizer(OptimizerMixin):
             wrapped_objective = objective_and_grad
             jac = None
 
-        minuit = iminuit.Minuit(wrapped_objective, init_pars, grad=jac)
-        minuit.errors = step_sizes
+        minuit = iminuit.Minuit(wrapped_objective, init_pars, grad=jac, name=par_names)
         minuit.limits = init_bounds
         minuit.fixed = fixed_bools
         minuit.print_level = self.verbose
