@@ -1,11 +1,11 @@
-from .paramsets import (
+from pyhf.parameters.paramsets import (
     paramset,
     unconstrained,
     constrained_by_normal,
     constrained_by_poisson,
 )
-from .utils import reduce_paramsets_requirements
-from .paramview import ParamViewer
+from pyhf.parameters.utils import reduce_paramsets_requirements
+from pyhf.parameters.paramview import ParamViewer
 
 __all__ = [
     'paramset',
