@@ -1,5 +1,5 @@
 """The probability density function module."""
-from . import get_backend
+from pyhf import get_backend
 
 __all__ = ["Independent", "Normal", "Poisson", "Simultaneous"]
 
@@ -170,10 +170,10 @@ class Independent(_SimpleDistributionMixin):
             >>> independent = pyhf.probability.Independent(poissons)
             >>> values = pyhf.tensorlib.astensor([8.0, 9.0])
             >>> independent.log_prob(values)
-            -4.262483801927939
+            -4.26248380...
             >>> broadcast_value = pyhf.tensorlib.astensor([11.0])
             >>> independent.log_prob(broadcast_value)
-            -4.347743645878765
+            -4.34774364...
 
         Args:
             value (:obj:`tensor` or :obj:`float`): The value at which to evaluate the distribution
