@@ -1,24 +1,23 @@
 import json
-import jsonschema
-import pkg_resources
-from pathlib import Path
 import yaml
 import click
 import hashlib
+from gettext import gettext
 
-from .exceptions import InvalidSpecification
+import sys
 
-SCHEMA_CACHE = {}
-SCHEMA_BASE = "https://scikit-hep.org/pyhf/schemas/"
-SCHEMA_VERSION = '1.0.0'
+# importlib.resources.as_file wasn't added until Python 3.9
+# c.f. https://docs.python.org/3.9/library/importlib.html#importlib.resources.as_file
+if sys.version_info >= (3, 9):
+    from importlib import resources
+else:
+    import importlib_resources as resources
 
 __all__ = [
     "EqDelimStringParamType",
     "citation",
     "digest",
-    "load_schema",
     "options_from_eqdelimstring",
-    "validate",
 ]
 
 
@@ -26,44 +25,6 @@ def __dir__():
     return __all__
 
 
-def load_schema(schema_id, version=None):
-    global SCHEMA_CACHE
-    if not version:
-        version = SCHEMA_VERSION
-    try:
-        return SCHEMA_CACHE[f'{SCHEMA_BASE}{Path(version).joinpath(schema_id)}']
-    except KeyError:
-        pass
-
-    path = pkg_resources.resource_filename(
-        __name__, str(Path('schemas').joinpath(version, schema_id))
-    )
-    with open(path) as json_schema:
-        schema = json.load(json_schema)
-        SCHEMA_CACHE[schema['$id']] = schema
-    return SCHEMA_CACHE[schema['$id']]
-
-
-# load the defs.json as it is included by $ref
-load_schema('defs.json')
-
-
-def validate(spec, schema_name, version=None):
-    schema = load_schema(schema_name, version=version)
-    try:
-        resolver = jsonschema.RefResolver(
-            base_uri=f"file://{pkg_resources.resource_filename(__name__, 'schemas/'):s}",
-            referrer=schema_name,
-            store=SCHEMA_CACHE,
-        )
-        validator = jsonschema.Draft6Validator(
-            schema, resolver=resolver, format_checker=None
-        )
-        return validator.validate(spec)
-    except jsonschema.ValidationError as err:
-        raise InvalidSpecification(err, schema_name)
-
-
 def options_from_eqdelimstring(opts):
     document = '\n'.join(
         f"{opt.split('=', 1)[0]}: {opt.split('=', 1)[1]}" for opt in opts
@@ -81,6 +42,24 @@ class EqDelimStringParamType(click.ParamType):
             self.fail(f'{value:s} is not a valid equal-delimited string', param, ctx)
 
 
+class VolumeMountPath(click.Path):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.name = f'{self.name}:{gettext("path")}'
+
+    def convert(self, value, param, ctx):
+        try:
+            path_host, path_mount = value.split(':')
+        except ValueError:
+            # too many values to unpack / not enough values to unpack
+            self.fail(f"{value!r} is not a valid colon-separated option", param, ctx)
+
+        return (
+            super().convert(path_host, param, ctx),
+            self.coerce_path_result(path_mount),
+        )
+
+
 def digest(obj, algorithm='sha256'):
     """
     Get the digest for the provided object. Note: object must be JSON-serializable.
@@ -132,7 +111,7 @@ def citation(oneline=False):
 
         >>> import pyhf
         >>> pyhf.utils.citation(oneline=True)
-        '@software{pyhf,  author = {Lukas Heinrich and Matthew Feickert and Giordon Stark},  title = "{pyhf: v0.6.2}",  version = {0.6.2},  doi = {10.5281/zenodo.1169739},  url = {https://doi.org/10.5281/zenodo.1169739},  note = {https://github.com/scikit-hep/pyhf/releases/tag/v0.6.2}}@article{pyhf_joss,  doi = {10.21105/joss.02823},  url = {https://doi.org/10.21105/joss.02823},  year = {2021},  publisher = {The Open Journal},  volume = {6},  number = {58},  pages = {2823},  author = {Lukas Heinrich and Matthew Feickert and Giordon Stark and Kyle Cranmer},  title = {pyhf: pure-Python implementation of HistFactory statistical models},  journal = {Journal of Open Source Software}}'
+        '@software{pyhf,  author = {Lukas Heinrich and Matthew Feickert and Giordon Stark},  title = "{pyhf: v0.7.0}",  version = {0.7.0},  doi = {10.5281/zenodo.1169739},  url = {https://doi.org/10.5281/zenodo.1169739},  note = {https://github.com/scikit-hep/pyhf/releases/tag/v0.7.0}}@article{pyhf_joss,  doi = {10.21105/joss.02823},  url = {https://doi.org/10.21105/joss.02823},  year = {2021},  publisher = {The Open Journal},  volume = {6},  number = {58},  pages = {2823},  author = {Lukas Heinrich and Matthew Feickert and Giordon Stark and Kyle Cranmer},  title = {pyhf: pure-Python implementation of HistFactory statistical models},  journal = {Journal of Open Source Software}}'
 
     Keyword Args:
         oneline (:obj:`bool`): Whether to provide citation with new lines (default) or as a one-liner.
@@ -140,14 +119,9 @@ def citation(oneline=False):
     Returns:
         citation (:obj:`str`): The citation for this software
     """
-    path = Path(
-        pkg_resources.resource_filename(
-            __name__, str(Path('data').joinpath('citation.bib'))
-        )
-    )
-    with path.open() as fp:
-        # remove end-of-file newline if there is one
-        data = fp.read().strip()
+    ref = resources.files('pyhf') / 'data' / 'citation.bib'
+    with resources.as_file(ref) as path:
+        data = path.read_text().strip()
 
     if oneline:
         data = ''.join(data.splitlines())
